# operations with temporal quantities (tau = 0)
# Vladimir Batagelj March 10, 2014

a = [(1,5,2), (6,8,1), (11,12,3), (14,16,2), (17,18,5), (19,20,1)]
b = [(2,3,4), (4,7,3), (9,10,2), (13,15,5), (16,21,1)]
inf = float("inf")
N = []
E = [(1,inf,1)]   # the third component is the unit in semiring

def get(S):
   try: return(next(S))
   except StopIteration: return((inf,inf,0))

def plus(a,b): return(a+b) 
def times(a,b): return(a*b) 

def standard(a):
   if len(a) == 0: return(a)
   fc = inf; c = []
   for (sa,fa,va) in a:
      if fc == sa:
         if vc == va: fc = fa
         else:
            c.append((sc,fc,vc))
            vc = va; sc = sa; fc = fa
      else:
         if fc != inf: c.append((sc,fc,vc))
         sc = sa; fc = fa; vc = va
   c.append((sc,fc,vc))
   return(c)

def binary(a):
   c = []
   for (sa,fa,va) in a:
      if va > 0: c.append((sa,fa,1))
   return(c)

def invert(a):
   c = []
   for (sa,fa,va) in a:
      c.append((sa,fa,1/va))
   return(c)

def central(v,C):
   cc = c = []; n = len(C)
   for t in range(n):
      if t!=v:
         c = sum(c,invert(C[v][t]))         
   for (s,f,v) in c: cc.append((s,f,v/(n-1)))
   return(cc)
                               
def sum(a,b):
   if len(a) == 0: return(b)
   if len(b) == 0: return(a)
   c = []; A = a.__iter__(); B = b.__iter__()
   (sa,fa,va) = get(A); (sb,fb,vb) = get(B)
   while (sa<inf) or (sb<inf):
      if sa < sb:
         sc = sa; vc = va
         if sb < fa: fc = sb; sa = sb
         else: fc = fa; (sa,fa,va) = get(A)        
         c.append((sc,fc,vc))
      elif sa == sb:
         sc = sa; fc = min(fa,fb); vc = plus(va,vb)
         c.append((sc,fc,vc))
         sa = sb = fc; fA = fa
         if fA <= fb: (sa,fa,va) = get(A)
         if fb <= fA: (sb,fb,vb) = get(B)
      else:
         sc = sb; vc = vb
         if sa < fb: fc = sa; sb = sa
         else: fc = fb; (sb,fb,vb) = get(B)        
         c.append((sc,fc,vc))
   return(standard(c))

def prod(a,b):
   if len(a)*len(b) == 0: return([])
   c = []; A = a.__iter__(); B = b.__iter__()
   (sa,fa,va) = get(A); (sb,fb,vb) = get(B)
   while (sa<inf) or (sb<inf):
      if fa <= sb: (sa,fa,va) = get(A)
      elif fb <= sa: (sb,fb,vb) = get(B)
      else:
         sc = max(sa,sb); fc = min(fa,fb); vc = times(va,vb)
         c.append((sc,fc,vc))
         if fc == fa: (sa,fa,va) = get(A)
         if fc == fb: (sb,fb,vb) = get(B)          
   return(standard(c))         

def listTemp(a):
   for (sa,fa,va) in a:
      c.append((sa,fa,1/va))
   return(c)

def listMatrix(M,names=None,all=True):
   n = len(M)
   for i in range(n):
      for j in range(n):
         if len(M[i][j])>0:
            if names==None: print('(',i+1,',',j+1,') =',M[i][j])
            else:
               print('(',names[i],',',names[j],') =\n  ',M[i][j])
         
def closure(R):
   n = len(R); C = R
   for k in range(n):
      for u in range(n):
         for v in range(n): 
            C[u][v] = sum(C[u][v], prod(C[u][k],C[k][v]))
      C[k][k] = sum(E,C[k][k])
   return(C)

def binMat(A):
   nr = len(A); nc = len(A[0])
   B = [[ [] for i in range(nc)] for j in range(nr)]
   for u in range(nr):
      for v in range(nc):
         B[u][v] = binary(A[u][v])
   return(B) 

def sumSyM(C,Rows,Cols,diag=False):
# extend for Rows and Cols are temporal partitions
   n = len(C); s = []
   for u in Rows:
      for v in Cols:
         if u>v: s = sum(s,C[v][u]) 
         elif u<v: s = sum(s,C[u][v]) 
         elif diag: s = sum(s,C[v][u])
   return(s)



                
